@@ -4,19 +4,39 @@ import requests
 
 __all__ = ['Resource', 'Collection']
 
+ALL_METHODS = 'HEAD, OPTIONS, GET, PUT, POST, DELETE'
+
+
+class SessionFactory(object):
+    default_headers = {
+        'Accept': 'application/json',
+        'Content-Type': 'application/json',
+        'Content-Length': '0',
+        'User-Agent': 'api_toolkit',
+    }
+
+    @classmethod
+    def make(cls, user, password):
+        session = requests.Session()
+        session.auth = (user, password)
+        session.headers.update(cls.default_headers)
+
+        return session
+
 
 class Resource(object):
     url_attribute_name = 'url'
     _session = None
+    session_factory = SessionFactory
 
     def __repr__(self):
-        return '<api_toolkit.Resource type="%s">' % self._type
+        return '<api_toolkit.Resource type="%s">' % self.__class__
 
     def __init__(self, data, **kwargs):
         self.resource_data = data
         self._links = kwargs.get('links', {})
-        self._type = kwargs.get('type', None)
         self._session = kwargs.get('session', self._session)
+        self._allowed_methods = kwargs.get('allowed_methods', ALL_METHODS)
 
         self.prepare_collections()
 
@@ -28,21 +48,13 @@ class Resource(object):
             user = kwargs.get('user', '')
             password = kwargs.get('password', '')
 
-            session = requests.Session()
-            session.auth = (user, password)
-            session.headers.update({
-                'Accept': 'application/json',
-                'Content-Type': 'application/json',
-                'Content-Length': '0',
-                'User-Agent': 'api_toolkit',
-            })
+            session = cls.session_factory.make(user, password)
 
         response = session.get(url)
         response.raise_for_status()
 
         instance = cls(
             data=response.json(),
-            type=kwargs.get('type'),
             links=response.links,
             session=session,
         )
@@ -84,12 +96,15 @@ class Resource(object):
             link_name = item['rel']
             link_url = item['url']
             link_collection = Collection(
-                link_url, type=link_name, session=self._session
+                link_url, session=self._session
             )
 
             setattr(self, link_name, link_collection)
 
     def save(self):
+        if 'PUT' not in self._allowed_methods:
+            raise ValueError('This resource cannot be saved.')
+
         dumped_data = json.dumps(self.resource_data)
         headers = {
             'Content-Length': str(len(dumped_data))
@@ -113,6 +128,9 @@ class Resource(object):
         return self
 
     def delete(self):
+        if 'DELETE' not in self._allowed_methods:
+            raise ValueError('This resource cannot be deleted.')
+
         headers = {}
         if self.etag:
             headers.update({'If-Match': self.etag})
@@ -121,39 +139,39 @@ class Resource(object):
 
 
 class Collection(object):
-    url = None
-    resource_class = Resource
-    _session = None
-    _type = 'entrypoint'
+    session_factory = SessionFactory
 
     def __repr__(self):
-        return '<api_toolkit.Collection type="%s">' % self._type
+        return '<api_toolkit.Collection type="%s">' % self.__class__
 
     def __init__(self, url, **kwargs):
         self.url = url
-        self._type = kwargs.get('type')
         self._session = kwargs.get('session')
+        self.resource_class = kwargs.get('resource_class', Resource)
 
         if self._session is None:
             user = kwargs.get('user', '')
             password = kwargs.get('password', '')
 
-            self._session = requests.Session()
-            self._session.auth = (user, password)
-            self._session.headers.update({
-                'Accept': 'application/json',
-                'Content-Type': 'application/json',
-                'Content-Length': '0',
-                'User-Agent': 'api_toolkit',
-            })
+            self._session = self.session_factory.make(user, password)
+
+        self._allowed_methods = self.discover_allowed_methods()
+
+    def discover_allowed_methods(self):
+        response = self._session.options(self.url)
+        return response.headers.get('Allow', ALL_METHODS)
 
     def all(self):
+        if 'GET' not in self._allowed_methods:
+            raise ValueError('This collection is not iterable.')
+
         url = self.url
         while True:
             response = self._session.get(url)
             for item in response.json():
                 instance = self.resource_class(
-                    data=item, type=self._type, session=self._session
+                    data=item, session=self._session,
+                    allowed_methods=response.headers.get('Allow', None),
                 )
                 yield instance
 
@@ -163,15 +181,21 @@ class Collection(object):
             url = response.links['next']['url']
 
     def get(self, identifier, append_slash=True):
+        if 'GET' not in self._allowed_methods:
+            raise ValueError('This collection cannot be loaded.')
+
         if append_slash:
             url_template = '{0}{1}/'
         else:
             url_template = '{0}{1}'
 
         url = url_template.format(self.url, identifier)
-        return self.resource_class.load(url, type=self._type, session=self._session)
+        return self.resource_class.load(url, session=self._session)
 
     def create(self, **kwargs):
+        if 'POST' not in self._allowed_methods:
+            raise ValueError('No items can be created for this collection.')
+
         resource_data = json.dumps(kwargs)
         response = self._session.post(
             self.url,
@@ -180,5 +204,18 @@ class Collection(object):
         )
 
         response.raise_for_status()
-        return self.resource_class.load(response.headers['Location'], type=self._type, session=self._session)
+        
+        try:
+            instance = self.resource_class(
+                data=response.json(),
+                links=response.links,
+                session=self._session,
+            )
+            instance.url = self.url
+        
+            instance._response = response
 
+        except ValueError:
+            instance = self.resource_class.load(response.headers['Location'], session=self._session)
+
+        return instance
